/**
 * DiXiM module contains fundamental things for DiXiM JavaScript library
 * such as core utility functions, variables, etc.
 *
 * @module dixim
 */

/**
 * Composes the dixim namespace.
 *
 * @class dixim
 * @static
 */
var dixim = dixim || {};

/**
 * Returns the namespace specified and creates it if it doesn't exist.
 *
 * The namespace is added under 'dixim' and the prefix 'dixim.'
 * can be omitted from <em>ns</em>. 
 *
 * @example
 *     var ns1 = dixim.namespace('foo.bar');
 *     var ns2 = dixim.namespace('dixim.foo.bar');
 *     ns1 === ns2; // true;
 *
 * @method namespace
 * @static
 * @param {String} ns namespace.
 * @return {Object} namespace object.
 */
dixim.namespace = function(ns) {
  var parts;
  var parent;
  var i;
  var len;
  var mod;

  parts = ns.split('.');
  if (parts[0] === 'dixim') {
    parts = parts.slice(1);
  }

  len = parts.length;
  for (parent = dixim, i = 0; i < len; i += 1) {
    mod = parts[i];
    if (typeof parent[mod] === 'undefined') {
      parent[mod] = {};
    }
    parent = parent[mod];
  }

  return parent;
};

/**
 * Checks whether the object is an Array.
 *
 * @method isArray
 * @static
 * @param {Object} obj
 * @return {Boolean} true if the <tt>obj</tt> is an Array, false otherwise.
 */
dixim.isArray = (function() {
  var toStr = Object.prototype.toString;
  var desc = '[object Array]';

  return function(obj) {
    return (typeof obj === 'object' && toStr.call(obj) === desc);
  };
}());

/**
 * Merges properties of an object into another object.
 *
 * The <tt>target</tt> will be modified.
 *
 * @example
 *     var target = {p1: 'test'};
 *     var obj = {p2: {p2c1: 'Hello'}};
 *     dixim.extend(target, obj); // shallow
 *
 *     // This affects the property of the target.
 *     obj.p2.p2c1 = 'Goodbye';
 *     console.log(target);
 *       // Object
 *       //  p1: "test"
 *       //  p2: Object
 *       //   p2c1: "Goodbye"
 *
 * @example
 *     var target = {p1: 'test',
 *                   p2: {p2c1: 'p2 child1', p2c2: [1, 2, 3, 4]},
 *                   p3: function() {alert('foo');}};
 *     var obj = {p1: 'overridden test',
 *                p4: {p4c1: 'imported prop'},
 *                p5: function() {console.log(this.p2);}};
 *     dixim.extend(target, obj, true); // deep
 *
 *     // This doesn't affect the property of the target.
 *     obj.p4.p4c1 = 'modified imported prop';
 *     console.log(target);
 *       // Object
 *       //  p1: "overridden test"
 *       //  p2: Object
 *       //   p2c1: "p2 child1"
 *       //   p2c2: Array[4]
 *       //  p3: function () {alert('foo');}
 *       //  p4: Object
 *       //   p4c1: "imported prop"
 *       //  p5: function () {console.log(this.p2);}
 *
 *     target.p5();
 *       // Object
 *       // p2c1: "p2 child1"
 *       // p2c2: Array[4]
 *
 * @method extend
 * @static
 * @param {Object} target an object to be extended.
 * @param {Object} obj an object to be merged into the <tt>target</tt>
 * @param {Boolean} [deep=false] if true, it does deep copying.
 * @return {Object} <tt>target</tt>
 */
dixim.extend = function(target, obj) {
  var isArray = dixim.isArray;
  var extend = dixim.extend;
  var deep = arguments[2] || false;
  var tmp;
  var i;

  for (i in obj) {
    if (!obj.hasOwnProperty(i)) continue;
    if (deep && typeof obj[i] === 'object') {
      tmp = isArray(obj[i]) ? [] : {};
      target[i] = extend(tmp, obj[i], true);
    } else {
      target[i] = obj[i];
    }
  }

  return target;
};

/**
 * Inherits the prototype properties from the constructor.
 *
 * @example
 *     var Parent = function() {this.parentProp = 'parent prop.';};
 *     Parent.prototype.getParentProp = function() {return this.parentProp;};
 *     var Child = function() {Parent.call(this);};
 *     dixim.inherit(Child, Parent);
 *     var c = new Child();
 *     c.getParentProp(); // parent prop
 *
 * @method inherit
 * @static
 * @param {Function} childConstructor
 * @param {Function} parentConstructor
 */
dixim.inherit = (function(){
  var tmpFn = function() {};
  return function(childConstructor, parentConstructor) {
    tmpFn.prototype = parentConstructor.prototype;
    childConstructor.superConstructor = parentConstructor;
    childConstructor.prototype = new tmpFn();
    childConstructor.prototype.constructor = childConstructor;
  };
}());

/**
 * Creates an object inherits properties from another object.
 *
 * @method create
 * @static
 * @param {Object} parent an object to be inherited.
 * @return {Object} new object that inherits <em>parent</em>.
 */
dixim.create = (function() {
  var tmpFn = function() {};
  return function(parent) {
    tmpFn.prototype = parent;
    return new tmpFn();
  };
}());

/**
 * Returns a function that calls the specified function
 * that has the specified context.
 *
 * @example
 *     var fn = function() {return this.test();};
 *     var ctx = {test: function() {console.log('hello')}};
 *     var proxy = dixim.proxy(fn, ctx);
 *
 *     fn(); // An exception occurs.
 *     proxy(); // "hello"
 *     fn.call(ctx); // equivalent to the above. "hello"
 *
 * @method proxy
 * @static
 * @param {Function} fn a function to be called.
 * @param {Object} context
 * @returns {Function}
 */
dixim.proxy = function(fn, context) {
  return function() {
    return fn.apply(context, arguments);
  };
};
