/**
 * @module ctt
 * @submodule content-list-table-view-formatter
 */
(function() {
var Y = dixim.ctt.YUI;
var Formatter = dixim.ctt.util.Formatter;
var Configs = dixim.ctt.Configs;
var ContentListTableViewFormatter;
var encodeId = dixim.xml.encodeId;
var escapeHtml = dixim.xml.escape;
var _ = dixim.ctt.ResourceBundle.get;

/**
 * Cell formatters for ContentListTableView.
 *
 * All formatters implement the interface of either
 * <em>formatter</em> or <em>nodeFormatter</em> that is defined in
 * [YUI.DataTable](http://yuilibrary.com/yui/docs/datatable/#formatters).
 * For the performance reason, almost all formatters are
 * <em>formatter</em> unless otherwise described.
 *
 * @class ContentListTableViewFormatter
 */
ContentListTableViewFormatter = {};
dixim.ctt.ContentListTableViewFormatter = ContentListTableViewFormatter;

var TEMPLATE_CELL_VALUE_HARNESS = '<div class="content-list-table-view-cell-bounds-harness">${cellContent}</div>';
var TEMPLATE_ROW_SELECTOR_CELL = '<input type="checkbox" value="${recordId}" name="rowSelector" />';
var TEMPLATE_TITLE_CELL = '${icon}${title}<input type="hidden" value="${recordId}" name="recordId" />';
var TEMPLATE_TITLE_CELL_MEDIA_ICON = '<img src="${path}" width="35" height="25" align="absmiddle" />';
var TEMPLATE_EMPTY_CELL = '&nbsp;';

/**
 * @method rowSelector
 * @static
 */
ContentListTableViewFormatter.rowSelector = function(o) {
  var record = o.record;
  var id = record.get('id');

  o.className += ' content-list-table-view-cell-row-selector';
  if (record.get('isStaticRow') || (record.get('writeStatus') === "PROTECTED")) return TEMPLATE_EMPTY_CELL;
  id = encodeId(id);
  return TEMPLATE_ROW_SELECTOR_CELL.replace('${recordId}', id);
};

/**
 * @method rowSelectorForDownloadMoveIn
 * @static
 */
ContentListTableViewFormatter.rowSelectorForDownloadMoveIn = function(o) {
  var record = o.record;
  var html;

  html = ContentListTableViewFormatter.rowSelector(o);
  if (!!!record.get('dn_movable')) return TEMPLATE_EMPTY_CELL;
  return html;
};

/**
 * @method title
 * @static
 */
ContentListTableViewFormatter.title = function(o) {
  var icon;
  var file;
  var cls;
  var mtype;
  var record = o.record;
  var title = o.value;
  var autoRemoveProtected;

  cls = record.get('class');
  file = Configs.get('imagePath') + '/';

  if (!!ContentListTableViewFormatter._isAutoDelete){
    autoRemoveProtected = record.get('autoRemoveProtected');
  }

  if (cls === 'parentContainer') {
    file += 'ico_dir.png';
  } else if (cls === 'container') {
    file += 'ico_folder.png';
  } else {
    mtype = record.get('mediaType');
    if (mtype === 'video') {
      if (autoRemoveProtected) {
        file += 'ico_tv_lock.png';
      } else {
        file += 'ico_tv.png';
      }
    } else if (mtype === 'audio') {
      file += 'ico_music.png';
    } else {
      file += 'ico_photo.png';
    }
  }
  icon = TEMPLATE_TITLE_CELL_MEDIA_ICON.replace('${path}', file);
  title = TEMPLATE_TITLE_CELL.
          replace('${recordId}', encodeId(record.get('id'))).
          replace('${icon}', icon).
          replace('${title}', escapeHtml(title));
  title = TEMPLATE_CELL_VALUE_HARNESS.replace('${cellContent}', title);
  o.className += ' content-list-table-view-cell-title';
  return title;
};

/**
 * @method compressState
 * @static
 */
ContentListTableViewFormatter.compressState = function(o) {
  var state;
  var compressState;

  state = o.data.atsTranscodeState;
  if (!!!state) {
    return TEMPLATE_EMPTY_CELL;
  }
  if (state === 4) {
    compressState = _('ctt_table_compression_state_label_finish');
  } else {
    compressState = _(['ctt_table_compression_state_label_',state].join(''));
  }
  return compressState;
};

/**
 * @method date
 * @static
 */
ContentListTableViewFormatter.date = function(o) {
  var dateStr;
  var date;

  if (!!!(dateStr = o.value)) return TEMPLATE_EMPTY_CELL;
  if (!!!(date = Formatter.dcDateToDate(dateStr))) return TEMPLATE_EMPTY_CELL;
  o.className += ' content-list-table-view-cell-date';
  return Formatter.stringifyDate(date, 'YYYY/MM/DD hh:mm:ss') || TEMPLATE_EMPTY_CELL;
};

/**
 * @method size
 * @static
 */
ContentListTableViewFormatter.size = function(o) {
  var size;

  o.className += ' content-list-table-view-cell-size';
  if (!!!(size = o.value)) return TEMPLATE_EMPTY_CELL;
  return Formatter.stringifySize(size);
};

/**
 * @method tags
 * @static
 */
ContentListTableViewFormatter.tags = (function() {
  var getSuffix = function(profile) {
    if (profile === 'DTCP_MPEG_TS_JP_T') {
      return 'HD';
    } else if (profile === 'DTCP_MPEG_TS_SD_JP_MPEG1_L2_T') {
      return 'SD';
    }
    return null;
  };

  return function(o) {
    var record = o.record;
    var tags = [];
    var suffix;
    var value;
    var comps;

    o.className += ' content-list-table-view-cell-tags';

    if (!!(value = record.get('profile'))) {
      comps = value.split('_');
      if (comps[0] === 'DTCP') {
        comps.shift();
      }
      suffix = getSuffix(value);
      value = !!suffix ? [comps[0], suffix].join(' ') : comps[0];
      tags.push('<div class="info">' + escapeHtml(value) + '</div>');
    }

    if (typeof (value = record.get('copyCount')) !== 'undefined') {
      if (value > 0) {
        tags.push('<div class="dubbing">COPY ' + value + '</div>');
      } else {
        tags.push('<div class="status">MOVE</div>');
      }
    }

    if (!!!(value = tags.join(''))) {
      return TEMPLATE_EMPTY_CELL;
    }
    return value;
  };
}());

}());
