/**
 * Holds utility codes.
 *
 * @module ctt
 * @submodule util
 */
(function() {
dixim.namespace('dixim.ctt.util');
}());

(function() {
/**
 * @class Formatter
 * @static
 */
var Formatter;

var pad2 = function(num) {
  var s = num.toString();
  if (s.length === 1) return ['0', s].join(''); 
  return s;
};

var getStringSize = function(str) {
  // Matches only the 10.. bytes that are non-initial characters in a multi-byte sequence.
  var ret = encodeURIComponent(str).match(/%[89ABab]/g);
  return str.length + (ret ? ret.length : 0);
};

Formatter = {
  /**
   * Stringifies Date object.
   *
   * YYYY/MM/DD hh:mm
   * YYYY/MM/DD hh:mm:ss
   *
   * @method stringifyDate
   * @static
   * @param {Date} d
   * @param {String} [f="YYYY/MM/DD hh:mm"] format. "YYYY/MM/DD hh:mm" | "YYYY/MM/DD hh:mm:ss"
   * @return {String}
   */
  stringifyDate: function(d, f) {
    var ymd;
    var time;

    ymd = [d.getFullYear(), pad2(d.getMonth() + 1), pad2(d.getDate())].join('/');
    time = [pad2(d.getHours()), pad2(d.getMinutes())];
    if (!!f && f === 'YYYY/MM/DD hh:mm:ss') {
      time.push(pad2(d.getSeconds()));
    }

    return [ymd, time.join(':')].join(' ');
  },

  /**
   * Converts dc:date format string to Date object.
   *
   * @method dcDateToDate
   * @static
   * @param {String} dcDate
   * @return {Date}
   */
  dcDateToDate: function(dcDate) {
    var ret;
    var date;
    var re = /^([0-9]{4})-([0-9]{2})-([0-9]{2})T([0-9]{2}):([0-9]{2}):([0-9]{2})(\.([0-9]{3}))?$/;

    if (!!!(ret = re.exec(dcDate))) return null;
    date = new Date(parseInt(ret[1], 10),
                    parseInt(ret[2], 10) - 1,
                    parseInt(ret[3], 10),
                    parseInt(ret[4], 10),
                    parseInt(ret[5], 10),
                    parseInt(ret[6], 10),
                    parseInt(ret[7], 10) || 0);
    return date;
  },

  /**
   * Converts Date object to dc:date format string.
   *
   * @method dateToDcDate
   * @static
   * @param {Date} d
   * @return {String}
   */
  dateToDcDate: function(d) {
    return [
      [d.getFullYear(), pad2(d.getMonth() + 1), pad2(d.getDate())].join('-'),
      'T',
      [pad2(d.getHours()), pad2(d.getMinutes()), '00'].join(':'),
      '.000'
     ].join('');
  },

  stringifySize: (function(){
    var i, KB, MB, GB, TB;
    i = 1000;
    KB = i;
    MB = KB * i;
    GB = MB * i;
    TB = GB * i;

    return function(value) {
      var size;

      if (typeof value === 'undefined') return '';
      if (!!!(size = parseInt(value, 10))) return '';
      if (size >= TB) {
        size = (size / TB).toFixed(1) + 'TB';
      } else if (size >= GB) {
        size = (size / GB).toFixed(1) + 'GB';
      } else if (size >= MB) {
        size = (size / MB).toFixed(1) + 'MB';
      } else if (size >= KB) {
        size = (size / KB).toFixed(1) + 'KB';
      } else {
        size = size + 'B';
      }
      return size;
    };
  }()),

  /**
   * Truncate string to specified length with ellipsis.
   *
   * @method truncateString
   * @param {String} s
   * @param {Number} len
   * @param {String} [ellipsis]
   * @returns
   */
  truncateString: function(s, len, ellipsis) {
    var n;

    if (s.length <= len) return s;
    if (!!ellipsis) {
      n = len - getStringSize(ellipsis);
    } else {
      n = len;
    }
    return [s.substring(0, n), ellipsis].join('');
  }
};
dixim.ctt.util.Formatter = Formatter;

}());

(function() {
var _ = dixim.ctt.ResourceBundle.get;

/**
 * Holds HTML form generators.
 *
 * @class Form
 */
var Form;

var TEMPLATE_TIME_SELECTOR = '${hour}${min}';

// TODO: when supporting the i18n, the order of the values
// should be localized accordingly.
Form = {
  /**
   * Creates time selector html.
   *
   * This method returns like the following two select HTML elements.
   *
   *     <!-- The "{suffix}" below will be replaced by
   *         'ctt_form_selector_time_hour' in ResourceBundle. -->
   *     <select name="hour">
   *       <option value="0" selected="selected">0{suffix}</option>
   *       <option value="1">1{suffix}</option>
   *       <option value="2">2{suffix}</option>
   *       <option value="3">3{suffix}</option>
   *       ...
   *       <option value="4">23</option>
   *     </select>
   *     <!-- The "{suffix}" below will be replaced by
   *         'ctt_form_selector_time_min' in ResourceBundle. -->
   *     <select name="min">
   *       <option value="0" selected="selected">00{suffix}</option>
   *       <option value="10">10{suffix}</option>
   *       <option value="20">20{suffix}</option>
   *       <option value="30">30{suffix}</option>
   *       <option value="40">40{suffix}</option>
   *       <option value="50">50{suffix}</option>
   *     </select>
   *
   * @method createTimeSelector
   * @param configs
   * @return {String}
   */
  createTimeSelector: function(configs) {
    var hourOpts = [];
    var minOpts = [];
    var i;
    var template;

    configs = configs || {};

    template = configs.template || TEMPLATE_TIME_SELECTOR;
    hourOpts.push('<select name="hour">');
    for (i = 0; i < 24; i += 1) {
      hourOpts.push([
        '<option value="', i, '">',
        _('ctt_form_selector_time_hour', i),
        '</option>'].join(''));
    }
    hourOpts.push('</select>');

    minOpts.push('<select name="min">');
    for (i = 0; i <= 50; i += 10) {
      minOpts.push([
        '<option value="', i, '">',
        _('ctt_form_selector_time_min', (i || '00')),
        '</option>'].join(''));
    }
    minOpts.push('</select>');

    return template.
           replace('${hour}', hourOpts.join('')).
           replace('${min}', minOpts.join(''));
  },

  /**
   * Creates date selector HTML.
   *
   * This method returns like the following HTML.
   * Each option@value is UNIX time.
   *
   *     <select name="date">
   *       <option value="1337266644857">2012/5/17</option>
   *       <option value="1337353044857">2012/5/18</option>
   *       <option value="1337439444857">2012/5/19</option>
   *       <option value="1337525844857">2012/5/20</option>
   *       <option value="1337612244857">2012/5/21</option>
   *       <option value="1337698644857">2012/5/22</option>
   *       <option value="1337785044857">2012/5/23</option>
   *     </select>
   *
   * @param configs
   * @return {String}
   */
  createDateSelector: function(configs) {
    var today = new Date();
    var dates;
    var oneday = 86400000;
    var dateOpts = [];
    var date;
    var time;
    var i;
    var length;

    configs = configs || {};
    length = configs.length || 7;

    today.setHours(0);
    today.setMinutes(0);
    today.setSeconds(0);
    today.setMilliseconds(0);
    dates = [today];
    time = today.getTime();
    for (i = 1; i < length; i += 1) {
      dates.push(new Date(time + oneday * i));
    }

    dateOpts.push('<select name="date">');
    for (i = 0; i < length; i += 1) {
      date = dates[i];
      dateOpts.push([
        '<option value="', date.getTime(), '">',
        [
         date.getFullYear(),
         ("00" + (date.getMonth() + 1)).slice(-2),
         ("00" + date.getDate()).slice(-2)
        ].join('/'),
        '</option>'
      ].join(''));
    }
    dateOpts.push('</select>');
    return dateOpts.join('');
  },
  /**
   * @method getNextDateRange
   * @param {Date} [date]
   * @return {Date}
   */
  getNextDateRange: function(date) {
    var min;
    var msec;
    var p;

    date = date || new Date();
    date = new Date(date.getTime() + 600000);
    date.setSeconds(0);
    date.setMilliseconds(0);
    min = date.getMinutes();
    msec = date.getTime();
    if (min && (p = min % 10)) {
      msec += 60000 * (10 - p);
    }
    return new Date(msec);
  }
};
dixim.ctt.util.Form = Form;

}());

(function() {
var Configs = dixim.ctt.Configs;

var Template;

Template = {
  LOADING_ICON_SMALL: [
    '<span class="ctt-loading ctt-loading-small">',
    '<img src="',
    Configs.get('imagePath'),
    '/loading_s.gif',
    '" /></span>'
  ].join(''),

  LOADING_ICON_LARGE: [
    '<span class="ctt-loading ctt-loading-large">',
    '<img src="',
    Configs.get('imagePath'),
    '/loading_l.gif',
    '" /></span>'
  ].join(''),
  LOADING_OVERLAY: '<div class="ctt-loading-overlay"></div>'
};

dixim.ctt.util.Template = Template;

}());
